#include "burnint.h"
// After Burner II - Run module

static unsigned char *Rom1, *Rom2;
static unsigned char *Tile;    static int nTileLen=0x40000;
static unsigned char *SprData; static int nSprDataLen=0x200000; // Sprite data
static unsigned char *RoadData;  static int nRoadDataLen=0x10000; // Ground Bitmap
static unsigned char *DrvPcm;

static unsigned char *Mem=NULL,*MemEnd=NULL;
static unsigned char *RamStart,*RamEnd;
static unsigned char *Ram0C,*Ram10,*Ram12,*Ram14,*Ram16,*Ram29,*Ram2E,*RamFF;
static unsigned int  *HighPal=NULL; static int nColCount=0x2000;

unsigned char AftGame=0; // 1=AB1, 2=AB2, 3=TB

unsigned char AftButton[10]={0,0,0,0,0,0,0,0,0,0};
unsigned char AftInput[6]={0,0,0,0x80,0x80,0x80};
unsigned char AftReset=0;

// This routine is called first to determine how much memory is needed (MemEnd-(unsigned char *)0),
// and then to set up all the pointers
static int MemIndex()
{
  unsigned char *Next; Next=Mem;
  Rom1 =Next;   Next+=0x080000;    // cpu1 program
  Rom2 =Next;   Next+=0x080000;    // cpu2 program
  Tile =Next;   Next+=nTileLen;    // tile data
  SprData=Next; Next+=nSprDataLen; // sprite data
  RoadData=Next;  Next+=nRoadDataLen;  // road bitmaps
  RamStart=Next;
  Ram0C=Next;   Next+=0x011000;   // 0C=tile screen
  Ram10=Next;   Next+=0x002000;   // 10=sprites
  Ram12=Next;   Next+=0x004000;   // 12=palette
  Ram29=Next;   Next+=0x008000;   // 29/2A=shared ram
  Ram2E=Next;   Next+=0x002004;   // +4 because it includes the 0ee001 flag
  RamFF=Next;   Next+=0x008000;
  AftZ80=Next;  Next+=0x010000; // z80 memory
  RamEnd=Next;
  DrvPcm=Next;  Next+=0x060000; // pcm data
  HighPal=(unsigned int *)Next; Next+=nColCount*sizeof(unsigned int);
  MemEnd=Next;
  return 0;
}

static void DrvSndCode(unsigned char nCode)
{
  // See if the code should be blocked
  if (BurnScodePass(nCode)==0) return;

  nAftSoundCode=nCode;
  ZetNmi();
  ZetRun(300); // run the z80 to accept the code
}

static int DrvSndGetBank (int nReg86) { return (nReg86>>4)&7; }

// Cpu 1 ----------------------------------
static unsigned char nInputSel=0;

static unsigned char Aft1ReadByte(unsigned int a)
{
  int ab; ab=(a>>16)&0xff;
  if (a==0x130001)
  {
    if (AftGame==3)
    {
      // Thunder Blade
      if (nInputSel==4)
      {
        if (AftButton[8]) return 0xff;
        if (AftButton[9]) return 0x00;
        return (unsigned char)(AftInput[5]);
      }
      if (nInputSel==8) return (unsigned char)(0xC0-(AftInput[4]>>1));
      return (unsigned char)(0x28+((0x100-AftInput[3])*0xb/0x10));
    }
    else
    {
      // After Burner
      if (nInputSel==4) return (unsigned char)(0xC0-(AftInput[4]>>1));
      if (nInputSel==8)
      {
        if (AftButton[8]) return 0xff;
        if (AftButton[9]) return 0x00;
        return (unsigned char)(AftInput[5]);
      }
      return (unsigned char)(0x28+(AftInput[3]*0xb/0x10));
    }
  }

  if (a==0x140001) return 0xff; // motor response?
  if (a==0x150001) return (unsigned char)(~AftInput[0]); //input
  if (a==0x150005) return (unsigned char)(~AftInput[1]); //dip 
  if (a==0x150007) return (unsigned char)(~AftInput[2]); //dip
  return 0;
}

static void Aft1WriteByte(unsigned int a,unsigned char d)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x12) { BsysPalWriteByte(a,d); return; } // palette write
  if (a==0x130001) { nInputSel=d; return; } // Input select
  if (a==0xe8017) { DrvSndCode(d); return; } // Send sound code
}

static unsigned short Aft1ReadWord(unsigned int a)
{
  int ab; ab=(a>>16)&0xff;
  // Handle the math chips:
  if (ab==0x0E) return CalcRead(0,a);
  if (ab==0x2E) return CalcRead(1,a);
  SEK_DEF_READ_WORD(a)
}

static void Aft1WriteWord(unsigned int a,unsigned short d)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x12) { BsysPalWriteWord(a,d); return; } // palette write
  // Handle the math chips:
  if (ab==0x0E) { CalcWrite(0,a,d); return; }
  if (ab==0x2E) { CalcWrite(1,a,d); return; }
  SEK_DEF_WRITE_WORD(a,d)
}

// Cpu 2 ----------------------------------
static unsigned short Aft2ReadWord(unsigned int a)
{
  int ab; ab=(a>>16)&0xff;
  // Handle the math chips:
  if (ab==0x0E) return CalcRead(1,a);
  SEK_DEF_READ_WORD(a)
}

static void Aft2WriteWord(unsigned int a,unsigned short d)
{
  int ab; ab=(a>>16)&0xff;
  // Handle the math chips:
  if (ab==0x0E) { CalcWrite(1,a,d); return; }
  SEK_DEF_WRITE_WORD(a,d)
}

static unsigned char Aft2ReadByte(unsigned int a)
{
  (void)a;
  return 0;
}

static void Aft2WriteByte(unsigned int a,unsigned char d)
{
  (void)a; (void)d;
}

static int AftResetCallback()
{
  // Reset instruction on cpu 1
  int nLast; nLast=nSekActive;
  SekClose();
  SekOpen(1); SekReset(); SekClose(); // Reset cpu 2
  SekOpen(nLast);
  return 0;
}

static int LoadRoms()
{
  int nRet=0; int i=0;
  // Load program roms
  nRet=BsysLoadCode(Rom1,0,4); if (nRet!=0) return 1;
  nRet=BsysLoadCode(Rom2,4,4); if (nRet!=0) return 1;

  // Load tile data (three bitfields)
  BsysLoadTiles(Tile,8);

  // Load the sprite roms and interleve by 4
  for (i=0;i<16;i++)
  {
    BurnLoadRom(SprData+0x80000*(i>>2)+(3-(i&3)),11+i,4);
  }

  // Load ground graphics
  BurnLoadRom(RoadData,27,1);

  // Load the z80 Rom
  BurnLoadRom(AftZ80,28,1);

  // Load PCM data
  memset(DrvPcm,0x80,0x60000); // fill sound with silence
  BurnLoadRom(DrvPcm+0x00000,29,1);
  BurnLoadRom(DrvPcm+0x20000,30,1);
  BurnLoadRom(DrvPcm+0x40000,31,1);
  return 0;
}

static int DrvReset()
{
  int i=0;
  for (i=0;i<2;i++) { SekOpen(i); SekReset(); SekClose(); } // Reset cpus
  ZetReset(); // reset z80
  return 0;
}

int AftInit()
{
  int nRet=0; int nLen=0;

  // Find out how much memory is needed
  Mem=NULL; MemIndex();
  nLen=MemEnd-(unsigned char *)0;
  Mem=(unsigned char *)malloc(nLen); if (Mem==NULL) return 1;
  memset(Mem,0,nLen); // blank all memory 
  MemIndex(); // Index the allocated memory

  nRet=LoadRoms(); if (nRet!=0) return 1; // Load the roms into memory

  SekInit(2); // Allocate 2 68000s
  SekExt[0].ResetCallback=AftResetCallback; // Get cpu 1 reset requests

  // Map in memory:
  // ----------------- Cpu 1 ------------------------
  SekOpen(0);
  SekMemory(Rom1 ,0x000000,0x07FFFF,SM_ROM); // cpu 1 Rom
  SekMemory(Ram0C,0x0C0000,0x0D0FFF,SM_RAM);
  SekMemory(Ram10,0x100000,0x101FFF,SM_RAM);
  SekMemory(Ram12,0x120000,0x123FFF,SM_ROM); // palette Ram (read-only - write goes through handler)
  SekMemory(Rom2 ,0x200000,0x27FFFF,SM_ROM); // cpu 2 Rom (visible from cpu 1)
  SekMemory(Ram29,0x29C000,0x2A3FFF,SM_RAM); // shared ram with cpu 2 (read/write)
  SekMemory(Ram2E,0x2EC000,0x2EE001,SM_RAM); // road
  SekMemory(RamFF,0xFF8000,0xFFFFFF,SM_RAM);
  SekExt[0].ReadByte =Aft1ReadByte;
  SekExt[0].WriteByte=Aft1WriteByte;
  SekExt[0].ReadWord =Aft1ReadWord;
  SekExt[0].WriteWord=Aft1WriteWord;
  SekClose();
  // ------------------------------------------------

  // ----------------- Cpu 2 ------------------------
  SekOpen(1);
  SekMemory(Rom2 ,0x000000,0x07FFFF,SM_ROM); // cpu 2 Rom
  SekMemory(Rom2 ,0x200000,0x27FFFF,SM_ROM); // (mirror)
  SekMemory(Ram29,0x09C000,0x0A3FFF,SM_RAM); // shared ram with cpu 1 (read/write)
  SekMemory(Ram29,0x29C000,0x2A3FFF,SM_RAM); // (mirror)
  SekMemory(Ram2E,0x0EC000,0x0EE001,SM_RAM); // road
  SekMemory(Ram2E,0x2EC000,0x2EE001,SM_RAM); // (mirror)
  SekExt[1].ReadWord =Aft2ReadWord;
  SekExt[1].WriteWord=Aft2WriteWord;
  SekExt[1].ReadByte =Aft2ReadByte;
  SekExt[1].WriteByte=Aft2WriteByte;
  SekClose();
  // ------------------------------------------------

  AftZInit(); // z80 init
  PcmRom=DrvPcm; PcmBankCount=6; PcmBankSize=16;  // bank length = 1<<16 = 0x10000
  PcmGetBank=DrvSndGetBank;
  AftSoundInit(); // Init sound emulation
  
  // ----------------- Draw init --------------------
  BsysPal=HighPal; BsysPalLen=nColCount; // palette (used for all graphics)
  BsysPram=Ram12; BsysPalInit();

  BsysTileData=Tile;  BsysTileDataLen=nTileLen; BsysTilePalBase=0x380; // Tiles
  BsysSprData=SprData; BsysSprDataLen=nSprDataLen; // Sprite bitmaps in Rom
  AftSprSrc=Ram10; // Sprite table in Ram
  // ------------------------------------------------

  DrvReset(); // Reset machine
  return 0;
}

int AftExit()
{

  // ----------------- Draw exit --------------------
  BsysPalExit();
  // ------------------------------------------------

  AftSoundExit(); // Exit sound emulation
  PcmRom=NULL; PcmBankCount=0; PcmBankSize=0;
  AftZExit(); // z80 exit

  SekExit(); // Deallocate 68000s
  // Deallocate all used memory
  if (Mem!=NULL) free(Mem);  Mem=NULL;

  AftGame=0; // Blank game to zero for safety
  return 0;
}

static int AftRoad()
{
  unsigned char *Road=Ram2E; // Road ram
  unsigned short *pr=NULL;
  unsigned char *pl=NULL; int y=0;

  pr=(unsigned short *)Road;
  for (pl=pBurnDraw,y=0; y<224; pr++,pl+=nBurnPitch,y++)
  {
    unsigned int c=0; int z=0;
    z=pr[0x000];
    c=BsysPal[0x1720];
    if (z&0x800) c=0;
    if (AftGame==3) c=0;
    BurnFillLine(pl,c);
  }
  return 0;
}

// Draw graphics
static int DrvDraw()
{
  BsysPalUpdate(); // Update the palette
  AftRoad();
  AftSprDraw();
  BsysTextPage(Ram0C);// Draw text
  return 0;
}

// Do one frame
int AftFrame()
{
  int nDone[3],nTotal[3],bSleep[2],i=0,nInterleve=50;
  short *pvf;

  if (AftReset) DrvReset(); // Reset machine

  pvf=(short *)(Ram29+0x7C00); // word(29C000+7C00) == -1,0 or 1 (to do with vblank?)

  // Compile separate buttons into AftInput[0]
  AftInput[0]=0; for (i=0;i<8;i++) AftInput[0]|=(AftButton[i]&1)<<i;

  nTotal[0]=12000000/60; // 12mhz 68000
  nTotal[1]=nTotal[0];
  nTotal[2]= 4000000/60; // 4 mhz z80
  nDone[0]=nDone[1]=nDone[2]=0;
  bSleep[0]=bSleep[1]=0; // 68000s aren't asleep yet

  AftSoundFrameStart(); // start rendering this frames's sound
  
  for (i=0;i<nInterleve;i++)
  {
    int j=0,nNext=0; // Next cycle count to run to
    int k=0;
    k=nInterleve-i-1;

    if ((k&1)==0)
    {
      // do io/sound update on cpu 1
      j=0; SekOpen(j); SekInterrupt(2); nDone[j]+=SekRun(500); SekClose();
    }

    // Run both 68000s
    for (j=0;j<2;j++)
    {
      int nInt=-1;
      if (i==0) nInt=4; // vint

      if (nInt>=0) bSleep[j]=0; // If we need to interrupt, wake the cpu up
      if (bSleep[j]) continue; // cpu is asleep

      SekOpen(j);
      if (nInt>=0) SekInterrupt(nInt); // Do intterupt

      nNext=(i+1)*nTotal[j]/nInterleve; // find out next cycle count to run to
      nDone[j]+=SekRun(nNext-nDone[j]);  // run cpu

      SekClose();

#if 1
      // Speed up cpus:
      {
        unsigned int pc; pc=SekPc(j);
        // If in a loop polling *pvf, so put to sleep for this frame
        if (AftGame==2)
        {
          if (j==0)
          {
            if (pc== 0xb676 || pc== 0xb67c) { if (*pvf> 0) bSleep[j]=1; } // bgt b676
            if (pc==0x14856 || pc==0x1485c) { if (*pvf>=0) bSleep[j]=1; }
          }
          else
          {
            if (pc==0x452 || pc==0x456) { if (*pvf> 0) bSleep[j]=1; } // bgt 452
            if (pc==0x556 || pc==0x55a) { if (*pvf>=0) bSleep[j]=1; } // bge 556
          }
        }
        else if (AftGame==1)
        {
          if (j==0)
          {
            if (pc==0x7f10 || pc==0x7f16) { if (*pvf> 0) bSleep[j]=1; }
            if (pc==0xe786 || pc==0xe78c) { if (*pvf>=0) bSleep[j]=1; }
          }
          else
          {
            if (pc==0x452 || pc==0x456) { if (*pvf> 0) bSleep[j]=1; }
            if (pc==0x536 || pc==0x53a) { if (*pvf>=0) bSleep[j]=1; }
          }
        }
      }
#endif

    }

    // Interleve the z80 a little bit (make sure we do it all though: hence 'k&7' not 'i&7' )
    j=2;
    if ((k&7)==0)
    {
      int nExe;
      nNext=(i+1)*nTotal[j]/nInterleve; // find out next cycle count to run to
      nExe=ZetRun(nNext-nDone[j]);   // run cpu
      nDone[j]+=nExe;
      nAftFmTimer-=nExe; // Move timer timeout back
    
      AftSoundRenderTo((i+1)*nBurnSoundLen/nInterleve); // Render sound for this portion
    }
  }

  AftSoundFrameStop(); // stop rendering this frames's sound

  if (pBurnDraw!=NULL) DrvDraw(); // Draw screen if needed
  return 0;
}

// Scan ram
int AftScan(int nAction,int *pnMin)
{
  struct BurnArea ba;

  if (nAction&4) // Scan volatile ram
  {
    if (pnMin!=NULL) *pnMin=0x009609; // Return minimum compatible version

    memset(&ba,0,sizeof(ba));
    ba.Data=RamStart;
    ba.nLen=RamEnd-RamStart;
    ba.szName="All Ram";
    BurnAcb(&ba);

    SekScan(nAction&3); // scan 68000 states

    CalcScan();

    AftZScan(nAction&3); // Scan AB Z80
    PcmScan(); // Scan PCM chip

    SCAN_VAR(nInputSel)
  }

  return 0;
}
