#include "burnint.h"
// After Burner II - z80

unsigned char *AftZ80=NULL;
unsigned char nAftSoundCode=0; // The sound code to pass to the z80 program
static int nTimerPeriod=0; // Period of the timer in cycles

int nAftFmTimer=0; // The z80 cycle count when FM timer A will carry out
static unsigned char FmReg[0x100]; // Our copy of the FM registers
static unsigned char nFmSel=0; // The FM register selected

static INLINE int CalcTimerPeriod()
{
  nTimerPeriod=(FmReg[0x10]<<2)|(FmReg[0x11]&3);
  nTimerPeriod=64*(1024-nTimerPeriod);
  return 0;
}

// Z80 accesses
static unsigned char AftZIn(unsigned short a)
{
  a&=0xff; // 8-bit ports
  if (a==1)
  {
    if (nAftFmTimer<=ZetMidCycles()) return 1; // timer happened
    return 0;
  }
  if (a==0x40)
  {
    return nAftSoundCode;
  }
  return 0;
}

static void AftZOut(unsigned short a,unsigned char d)
{
  a&=0xff;

  if (a==0) { nFmSel=d; return; } // FM Register select
  if (a==1)
  {
    // FM Register write
    FmReg[nFmSel]=d;
    // Write the register to the chip emu
    if (bAftSoundOkay) YM2151WriteReg(0,nFmSel,d);

    if (nFmSel==0x10 || nFmSel==0x11) { CalcTimerPeriod(); } // recalc timer period
    if (nFmSel==0x14 && (d&0x10))
    {
      int nNow=0; nNow=ZetMidCycles();
      // reset timer A - project next timer
      if (nTimerPeriod>0)
      {
        while (nAftFmTimer<=nNow) nAftFmTimer+=nTimerPeriod;
      }
    }
    return;
  }

}

int AftZInit()
{
  memset(FmReg,0,sizeof(FmReg));
  // Init the z80
  ZetIn=AftZIn; ZetOut=AftZOut;
  ZetRead=PcmRead; ZetWrite=PcmWrite;

  ZetInit();
  ZetMapArea    (0x0000,0xefff,0,AftZ80+0x0000); // Direct Read from ROM
  ZetMapArea    (0x0000,0xefff,2,AftZ80+0x0000); // Direct Fetch from RoM
  ZetMemCallback(0xf000,0xf0ff,0); // Pcm Read
  ZetMemCallback(0xf000,0xf0ff,1); // Pcm Write
  ZetMapArea    (0xf100,0xffff,0,AftZ80+0xf100); // Direct Read from RAM
  ZetMapArea    (0xf100,0xffff,1,AftZ80+0xf100); // Direct Write to RAM
  ZetMapArea    (0xf100,0xffff,2,AftZ80+0xf100); // Direct Fetch from RAM
  ZetMemEnd();

  // remember to do ZetReset() in main driver
  return 0;
}

int AftZExit()
{
  ZetExit();
  ZetRead=NULL; ZetWrite=NULL;
  ZetIn=NULL; ZetOut=NULL;
  memset(FmReg,0,sizeof(FmReg));
  return 0;
}

int AftZScan(int nAction)
{
  int i=0;
  SCAN_VAR(nAftFmTimer)
  SCAN_VAR(nAftSoundCode)
  SCAN_VAR(nFmSel)
  SCAN_VAR(FmReg)
  ZetScan(nAction); // Scan Z80

  if (nAction&2)
  {
    CalcTimerPeriod(); // If write, timer registers could have changed
    // Resend all the registers to the ym2151
    if (bAftSoundOkay) for (i=0;i<0x100;i++) { YM2151WriteReg(0,i,FmReg[i]); }
  }
  return 0;
}
