#include "burnint.h"
// Out Run - Init

static unsigned char *Data=NULL;

static int DrvSndGetBank(int Reg86)
{
  return (Reg86>>4)&7;
}

int OutInit()
{
  int Ret=0; int i=0;
  int DataLen=0;
  int x=0,y=0;
  unsigned char *pb=NULL,*po=NULL; // Pointer to bitmap and one-bit version

  // Init memory and cpus
  Ret=OutMemInit(); if (Ret!=0) return 1;

  // Load program roms and byteswap
  Ret=BsysLoadCode(OutRom1,0,4); if (Ret!=0) return 1;
  Ret=BsysLoadCode(OutRom2,4,4); if (Ret!=0) return 1;

  // Allocate space for graphics
  DataLen=0x144000+0x20000+0x30000; // Sprites, Tiles, High Palette; Ground; Sound;
  Data=(unsigned char *)malloc(DataLen); if (Data==NULL) return 1;
  memset(Data,0,DataLen);

  // ------------  Sprites  -----------------------------------------
  BsysSprData=Data; BsysSprDataLen=0x100000;
  // Load the roms and interleve by 4
  for (i=0;i<8;i++) BurnLoadRom(BsysSprData+0x80000*(i>>2)+(i&3),14+i,4);

  // ------------  Tiles  -------------------------------------------
  BsysTileData=Data+0x100000; BsysTileDataLen=0x40000;
  BsysTilePalBase=0;
  // Load tile data (three bitfields)
  BsysLoadTiles(BsysTileData+0x00000, 8);
  BsysLoadTiles(BsysTileData+0x20000,11);

  // ------------  Palette  -----------------------------------------
  BsysPal=(unsigned int *)(Data+0x140000); BsysPalLen=0x1000;
  BsysPram=OutRam12;
  BsysPalInit();

  // ------------  Ground   -----------------------------------------
  OutGrBmap=Data+0x144000;
  BurnLoadRom(OutGrBmap+0x20000,22,1);  // Load 1-bit version in pcm (borrow the memory)
  OutGrRam =OutRam28;
  OutGrFlag=OutRam29;

  // Decode the 0x200*0x100  1-bit,2-bitplane  bitmap into 8-bit
  pb=OutGrBmap; po=OutGrBmap+0x20000;
  for (y=0;y<0x100;y++)
  {
    int Pixel=0,Last=0;
    for (x=0; x<0x200; x+=8,pb+=8,po++)
    {
      unsigned int a; int c;
      a=po[0]|(po[0x4000]<<8);

      for (c=0;c<8;c++)
      {
        int Val; int Add;
        switch (a&0x8080)
        {
          default:     Val=3; break;
          case 0x0080: Val=2; break;
          case 0x8000: Val=1; break;
          case 0x8080: Val=0; break;
        }
        a<<=1;

        // Get rid of the line down the middle of the road
        pb[c]=(unsigned char)Pixel;
        Add=Val-Last;
             if (Last==3 && Val==0) { Add=0; }
        else if (Last==0 && Val==3) { Add=0; }
        else if (Last==2 && Val==0) { Add=1; }
        else if (Last==0 && Val==2) { Add=-1; }
        Pixel+=Add;

        Last=Val;
      }
    }
  }

  // ------------  Sound  -------------------------------------------

  AftZ80=OutZ80;
  AftZInit(); // z80 init
  PcmRom=Data+0x144000+0x20000; PcmBankCount=6; PcmBankSize=15; // 0x30000
  memset(PcmRom,0x80,0x30000); // fill sound with silence
  for (i=0;i<6;i++) BurnLoadRom(PcmRom+(i<<15),24+i,1);
  PcmGetBank=DrvSndGetBank;
  Ret=BurnLoadRom(OutZ80,23,1); if (Ret!=0) return 1;
  AftSoundInit(); // Init sound emulation

  OutRunReset(); // Reset machine

  return 0;
}

int OutExit()
{

  AftSoundExit();
  PcmRom=NULL; PcmBankCount=0; PcmBankSize=0;
  AftZExit();
  AftZ80=NULL;

  OutGrBmap=NULL;

  BsysPalExit();
  BsysPram=NULL; BsysPal=NULL; BsysPalLen=0;
  BsysTileData=NULL; BsysTileDataLen=0;
  BsysSprData=NULL; BsysSprDataLen=0;
  free(Data); Data=NULL;

  OutMemExit();
  return 0;
}
